﻿class Program : Config
{
    static void Main(string[] args)
    {
            
        loadConfig();
        run();
            
        var trigger = new Trigger(18,30); 
        trigger.OnTimeTriggered += () =>
        {
            run();
        };

        Thread.Sleep(Timeout.Infinite);
    }

    public static void run() {

        Console.WriteLine("Start Process");

        Console.WriteLine($"Found {config.sources.Count} Sources.");

        resetProjects();

        foreach (Source s in config.sources)
        {
            Console.WriteLine($"Found Source Type {s.type}.");
            switch (s.type)
            {
                case "gitlab":

                    var task_gitlab = Task.Run(() => {
                        GitlabCrawler g = new GitlabCrawler(config, s);
                    });

                    task_gitlab.ContinueWith(t => {
                        Console.WriteLine(t.Exception);
                    }, TaskContinuationOptions.OnlyOnFaulted);

                    break;
                case "folder":

                    var task_folder = Task.Run(() => {
                        FolderCrawler f = new FolderCrawler(config, s);
                    });

                    task_folder.ContinueWith(t => {
                        Console.WriteLine(t.Exception);
                    }, TaskContinuationOptions.OnlyOnFaulted);

                    break;
                default:
                    throw new Exception($"Source Type {s.type} not avaliable");
            }
        }
    }

    public static void resetProjects()
    {
        DirectoryInfo dir_html = new DirectoryInfo(Config.getConfig.html_output);
        DirectoryInfo dir_media = new DirectoryInfo(Config.getConfig.media_output);

        foreach (FileInfo file in dir_html.GetFiles())
        {
            file.Delete();
        }

        foreach (DirectoryInfo dir in dir_media.GetDirectories())
        {
            dir.Delete(true);
        }

        SQLDriver mysql_connection = new SQLDriver();
        mysql_connection.ExecuteWithoutResult($"Delete from projects;");
    }
}
