﻿using Newtonsoft.Json;


class Folder : CrawlerSource
{

    public Folder(string options_path, string project_path) : base()
    {

        Console.WriteLine($"Start import of Folder Project {project_path}.");

        this.options_path = options_path;
        this.project_path = project_path;

        loadOptions();

        loadPublicFile();
        savePublicFile();

        loadMediaFilesPaths();
        copyMediaFiles();

        saveToDB();
    }

    public void loadOptions()
    {
        using (StreamReader r = new StreamReader(options_path))
        {
            string json = r.ReadToEnd();
            options_content = JsonConvert.DeserializeObject<OptionsJSON>(json);

            if (options_content.type == "markdown")
            {
                public_file_path = project_path + Path.DirectorySeparatorChar + Config.getConfig.public_file + ".md";
            }
            else if (options_content.type == "html")
            {
                public_file_path = project_path + Path.DirectorySeparatorChar + Config.getConfig.public_file + ".html";
            }

            if (public_file_path != null)
            {
                this.last_activity = File.GetLastWriteTime(public_file_path).ToString("yyyy-MM-ddTHH:mm:ss");
            }

        }
    }

    private Boolean publicFileExists()
    {
        if (File.Exists(public_file_path))
        {
            return true;
        }
        Console.WriteLine($"Public File for: {options_path} does not exists.");
        return false;
    }

    public void loadPublicFile()
    {
        if (publicFileExists())
        {
            using (StreamReader r = new StreamReader(public_file_path))
            {
                string file = r.ReadToEnd();
                public_file_content = file;
                public_file_parsed_content = parsePublicFile();
            }
        }
    }

    public void loadMediaFilesPaths()
    {
        string[] dirs = Directory.GetFiles(project_path + Path.DirectorySeparatorChar + "media", "*");
        media_file_paths = dirs;
    }

    public void copyMediaFiles()
    {

        string media_output_folder = Config.getConfig.media_output + Path.DirectorySeparatorChar + options_content.project_alias_name;

        if (!Directory.Exists(media_output_folder))
        {
            Directory.CreateDirectory(media_output_folder);
        }

        foreach (string s in media_file_paths)
        {
            string filename = Path.GetFileName(s);
            string to_file = media_output_folder + Path.DirectorySeparatorChar + filename;
            if (!File.Exists(to_file))
            {
                File.Copy(s, to_file);
            }
        }
    }
}
