﻿using Newtonsoft.Json;
using System.Text;
using System.Text.RegularExpressions;
 
class Gitlab : CrawlerSource
{
    int project_id;
    Source s;

    public Gitlab(int project_id, Source s, Project p)
    {
        Console.WriteLine($"Start import of Gitlab Project {p.name}.");

        this.project_id = project_id;
        this.project_path = "";
        this.s = s;

        Regex rx = new Regex(@"^(.*?)\.");
        MatchCollection matches = rx.Matches(p.last_activity_at);

        if (matches.Count > 0 && matches[0].Groups.Count > 1)
        {
            this.last_activity = matches[0].Groups[1].Value;
        }

        loadOptions();

        if (options_content.load_tags)
        {
            loadTags();
        }

        loadPublicFile();
        savePublicFile();

        loadMediaFilesPaths();
        copyMediaFiles();

        saveToDB();
    }

    private void loadOptions()
    {

        string content = getGitlabFileContent("options.json");
        string json = Encoding.UTF8.GetString(Convert.FromBase64String(content));
        options_content = JsonConvert.DeserializeObject<OptionsJSON>(json);

        if (options_content.type == "markdown")
        {
            public_file_path = "public.md";
        }
        else if (options_content.type == "html")
        {
            public_file_path = "public.html";
        }

    }

    private string getGitlabFileContent(string file)
    {

        Dictionary<string, string> parameter = new Dictionary<string, string>() {
                {"ref", "master"},
                {"access_token", s.token}
            };

        string folder_spacer = "%2F";
        string replaced_file = file.Replace("/", folder_spacer);

        string? return_value = APIInterface.Get($"{s.path}/api/v4/projects/{project_id}/repository/files/{s.public_folder}{folder_spacer}{replaced_file}", parameter).Result;
        GitlabFile? gitlab_file = JsonConvert.DeserializeObject<GitlabFile>(return_value);

        return gitlab_file.content;
    }

    public void loadPublicFile()
    {
        string json = Encoding.UTF8.GetString(Convert.FromBase64String(getGitlabFileContent(public_file_path)));
        public_file_content = json;
        public_file_parsed_content = parsePublicFile();
    }

    public void loadMediaFilesPaths()
    {

        Dictionary<string, string> gitlab_folder_parameter = new Dictionary<string, string>() {
                            {"path", s.public_folder+"/media"},
                            {"access_token", s.token}
                        };

        string? gitlab_folder = APIInterface.Get($"{s.path}/api/v4/projects/{project_id}/repository/tree", gitlab_folder_parameter).Result;
        List<GitlabFolder>? gitlab_folder_list = JsonConvert.DeserializeObject<List<GitlabFolder>>(gitlab_folder);
        media_file_paths = gitlab_folder_list.Select(d => d.name).ToArray();
    }

    public void copyMediaFiles()
    {

        string media_output_folder = Config.getConfig.media_output + Path.DirectorySeparatorChar + options_content.project_alias_name;

        if (!Directory.Exists(media_output_folder))
        {
            Directory.CreateDirectory(media_output_folder);
        }

        foreach (string s in media_file_paths)
        {
            string p = getGitlabFileContent($"media/{s}");
            string to_file = media_output_folder + Path.DirectorySeparatorChar + s;
            File.WriteAllBytes(to_file, Convert.FromBase64String(p));
        }
    }

    public void loadTags()
    {
        Dictionary<string, string> parameter = new Dictionary<string, string>() {
                {"access_token", s.token}
            };

        string? return_value = APIInterface.Get($"{s.path}/api/v4/projects/{project_id}/repository/tags", parameter).Result;

        if (return_value != "[]")
        {

            string tags_output_folder = Config.getConfig.media_output + Path.DirectorySeparatorChar + options_content.project_alias_name + Path.DirectorySeparatorChar + "tags";

            if (!Directory.Exists(tags_output_folder))
            {
                Directory.CreateDirectory(tags_output_folder);
            }

            List<Tags>? tags_list = JsonConvert.DeserializeObject<List<Tags>>(return_value);

            addAdditionalMardonContent("## Tags");

            for (int i = 0; i < tags_list.Count; i++)
            {

                byte[] tag_file;

                if (
                    this.options_content.build_job_name != null &&
                    this.options_content.build_job_name != ""
                )
                {
                    Dictionary<string, string> parameter_get_tag = new Dictionary<string, string>() {
                        {"access_token", s.token},
                        {"job",  this.options_content.build_job_name}
                    };
                    tag_file = APIInterface.GetBinary($"{s.path}/api/v4/projects/{this.project_id}/jobs/artifacts/{tags_list[i].name}/download", parameter_get_tag).Result;
                }
                else {
                    Dictionary<string, string> parameter_get_tag = new Dictionary<string, string>() {
                        {"access_token", s.token},
                        {"sha",  tags_list[i].name}
                    };
                    tag_file = APIInterface.GetBinary($"{s.path}/api/v4/projects/{project_id}/repository/archive.zip", parameter_get_tag).Result;
                }


                if (tag_file != null) { 
                    string tag_file_output = $"{tags_output_folder}{Path.DirectorySeparatorChar}{tags_list[i].name}.zip";
                    File.WriteAllBytes(tag_file_output, tag_file);

                    addAdditionalMardonContent($"- [{tags_list[i].name}]({getMediaPath()}/tags/{tags_list[i].name}.zip)\n");                
                }

            }
        }
    }
}